
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

/*
 * Must be run with 32-bit JRE to link with ClassLibrary1.dll
 */
public class Program {

	static {
		//Path to ClassLibrary1.dll. ClassLibrary1 solution must be compiled before running this.
		File cl1Dll = new File("../../Java/ClassLibrary1/Debug/ClassLibrary1.dll");
		String pathToJNI = "";
		try {
			pathToJNI = cl1Dll.getCanonicalPath();
		} catch (IOException e1) {
			e1.printStackTrace();
			System.exit(1);
		}
		try {
	    	System.load(pathToJNI);
	    } catch (UnsatisfiedLinkError e) {
	      System.err.println("Native code library failed to load.\n" + e);
	      System.exit(1);
	    }
	  }
	
	public native String Execute(String a);
	
	private static JTextField authKeyField;
	private static JTextField terminalIDField;
	private static JTextField xWebIDField;
	private static JTextArea resultsField;
	
	private static String getPinPadPromptXML() {
		return "<XLINKEMVREQUEST>" +
        "<TRANSACTIONTYPE>PPDEDITSETUP</TRANSACTIONTYPE>" +
      "</XLINKEMVREQUEST>";
	}
	
	private static String getPromptForSignatureXML() {
		return "<XLINKEMVREQUEST>" +
        "<TRANSACTIONTYPE>PPDPROMPTSIGNATURE</TRANSACTIONTYPE>" +
        "<TITLE>Please Sign</TITLE>" +
        "<DISPLAYCAPTUREDSIGNATURE>T</DISPLAYCAPTUREDSIGNATURE>" +
      "</XLINKEMVREQUEST>";
	}
	
	private static String getCreditSaleXML() {
		return String.format("<XLINKEMVREQUEST>" +
                                      "<TRANSACTIONTYPE>CREDITSALE</TRANSACTIONTYPE>" +
                                      "<XWEBAUTHKEY>%s</XWEBAUTHKEY>" +
                                      "<XWEBTERMINALID>%s</XWEBTERMINALID>" +
                                      "<XWEBID>%s</XWEBID>" +
                                      "<ALLOWDUPLICATES />" + 
                                      "<AMOUNT>2.09</AMOUNT>" +
                                    "</XLINKEMVREQUEST>", 
                                    authKeyField.getText(),
                                    terminalIDField.getText(),
                                    xWebIDField.getText());
	}
	
	private static void pinPadSetupAction() {
		Program p = new Program();
		String params = getPinPadPromptXML();
		String results = p.Execute(params);
		resultsField.setText(results);
	}
	
	private static void promptForSignatureAction() {
		Program p = new Program();
		String params = getPromptForSignatureXML();
		String results = p.Execute(params);
		resultsField.setText(results);
	}
	
	private static void creditSaleAction() {
		Program p = new Program();
		String params = getCreditSaleXML();
		String results = p.Execute(params);
		resultsField.setText(results);
	}

	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setLocation(500, 300);
		frame.setSize(500, 350);
		Container contents = frame.getContentPane();
		contents.setLayout(new BoxLayout(contents, BoxLayout.PAGE_AXIS));
		
		//Initalize field components
		authKeyField = new JTextField();
		terminalIDField = new JTextField();
		xWebIDField = new JTextField();
		
		//Inputs
		JPanel inputPanel = new JPanel();
		GridLayout inputPanelLayout = new GridLayout(3,2);
		inputPanel.setLayout(inputPanelLayout);
		inputPanel.add(new JLabel("XWeb auth key:"));
		inputPanel.add(authKeyField);
		inputPanel.add(new JLabel("XWeb Terminal ID:"));
		inputPanel.add(terminalIDField);
		inputPanel.add(new JLabel("XWeb ID:"));
		inputPanel.add(xWebIDField);
		
		//Buttons
		JButton pinPadSetup = new JButton("PinPad Setup");
		JButton promptForSignature = new JButton("Prompt for Signature");
		JButton creditSale = new JButton("Credit Sale");
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.LINE_AXIS));
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(15,15,15,15));
		buttonPanel.add(Box.createRigidArea(new Dimension(10, 10)));
		buttonPanel.add(pinPadSetup);
		buttonPanel.add(Box.createRigidArea(new Dimension(10, 10)));
		buttonPanel.add(promptForSignature);
		buttonPanel.add(Box.createRigidArea(new Dimension(10, 10)));
		buttonPanel.add(creditSale);
		
		//Add action listeners to buttons
		pinPadSetup.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				pinPadSetupAction();
			}
		});
		promptForSignature.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				promptForSignatureAction();
			}
		});
		creditSale.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				creditSaleAction();
			}
		});
		
		//Results
		resultsField = new JTextArea(5,20);
		JScrollPane scrollResults = new JScrollPane(resultsField);
		JPanel resultsPanel = new JPanel();
		resultsPanel.setLayout(new BoxLayout(resultsPanel, BoxLayout.Y_AXIS));
		JLabel resultsFieldLabel = new JLabel("Results:");
		resultsPanel.add(new JLabel("Results:"));
		resultsPanel.add(scrollResults);
		resultsFieldLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		resultsField.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		//Add all panels to contents
		contents.add(inputPanel);
		contents.add(buttonPanel);
		contents.add(resultsPanel);
		
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
		
	}
}
